/* 
	flap ?
*/

// -------------------------------------------------------------------------------------------------------------

#include "ext.h"							
#include "ext_obex.h"						
#include "jpatcher_api.h"
#include "jgraphics.h"

// -------------------------------------------------------------------------------------------------------------

typedef struct _flap {
	t_jbox		box ;						
	t_jrgba		couleur ;
	t_object	*mypatcher ;
	t_object	*mypatcherview ;
	void		*m_out ;
	} t_flap ;

// -------------------------------------------------------------------------------------------------------------

void *flap_new			(t_symbol *s, long argc, t_atom *argv) ;
void flap_free			(t_flap *x) ;
void flap_assist		(t_flap *x, void *b, long m, long a, char *s) ;

void flap_paint			(t_flap *x, t_object *patcherview) ; 
void flap_getdrawparams (t_flap *x, t_object *patcherview, t_jboxdrawparams *params) ;

void flap_mousedown		(t_flap *x, t_object *patcherview, t_pt pt, long modifiers) ;
void flap_mousedrag		(t_flap *x, t_object *patcherview, t_pt pt, long modifiers) ;
void flap_mouseup		(t_flap *x, t_object *patcherview, t_pt pt, long modifiers) ;
void flap_mouseenter	(t_flap *x, t_object *patcherview, t_pt pt, long modifiers) ;
void flap_mouseleave	(t_flap *x, t_object *patcherview, t_pt pt, long modifiers) ;
void flap_mousemove		(t_flap *x, t_object *patcherview, t_pt pt, long modifiers) ;

void flap_key			(t_flap *x, t_object *patcherview, long keycode, long modifiers, long textcharacter) ;
void flap_focusgained	(t_flap *x, t_object *patcherview) ;
void flap_focuslost		(t_flap *x, t_object *patcherview) ;

void flap_bang			(t_flap *x) ;

void flap_attach		(t_flap *x) ;
void flap_notify		(t_flap *x, t_symbol *s, t_symbol *msg, void *sender, void *data) ;

// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

static t_class *s_flap_class ;

int main (void)
	{	
		t_class *c ;
			
		c = class_new ("flap", (method)flap_new, (method)flap_free, sizeof(t_flap), 0L, A_GIMME, 0) ;

		c->c_flags |= CLASS_FLAG_NEWDICTIONARY ;
		jbox_initclass (c, JBOX_FIXWIDTH | JBOX_COLOR) ;

		class_addmethod	(c, (method)flap_assist,		"assist", A_CANT, 0) ;
		
		class_addmethod (c, (method)jbox_notify,		"notify", A_CANT, 0) ; 
		class_addmethod (c, (method)flap_notify,		"notify", A_CANT, 0) ; 
		class_addmethod	(c, (method)flap_paint,			"paint", A_CANT, 0) ;
		class_addmethod (c, (method)flap_getdrawparams, "getdrawparams", A_CANT, 0);
		
		class_addmethod	(c, (method)flap_mousedown,		"mousedown", A_CANT, 0) ;
		class_addmethod	(c, (method)flap_mousedrag,		"mousedrag", A_CANT, 0) ;
		class_addmethod	(c, (method)flap_mouseup,		"mouseup", A_CANT, 0) ;
		class_addmethod	(c, (method)flap_mouseenter,	"mouseenter", A_CANT, 0) ;
		class_addmethod	(c, (method)flap_mouseleave,	"mouseleave", A_CANT, 0) ;
		class_addmethod	(c, (method)flap_mousemove,		"mousemove", A_CANT, 0) ;
		
		class_addmethod (c, (method)flap_key,			"key", A_CANT, 0) ;
		class_addmethod (c, (method)flap_focusgained,	"focusgained", A_CANT, 0) ;
		class_addmethod (c, (method)flap_focuslost,		"focuslost", A_CANT, 0) ;
		
		class_addmethod	(c, (method)flap_bang,			"bang", 0) ;
		
		CLASS_ATTR_RGBA						(c, "color", 0, t_flap, couleur) ; 
		CLASS_ATTR_DEFAULTNAME_SAVE_PAINT	(c, "color", 0, "1. 0.75 0. 1.") ; 
		CLASS_ATTR_STYLE_LABEL				(c, "color", 0, "rgba", "Toto") ;
		CLASS_ATTR_CATEGORY					(c, "color", 0, "Behavior");
		
		CLASS_ATTR_DEFAULT					(c, "patching_rect", 0, "0. 0. 100. 100.") ;
		
		class_register (CLASS_BOX, c) ;
		s_flap_class = c ;

		return MAX_ERR_NONE ;
	}

// -------------------------------------------------------------------------------------------------------------

void *flap_new (t_symbol *s, long argc, t_atom *argv)
	{
		t_flap			*x = NULL ;
		t_dictionary	*d = NULL ;
		long			boxflags ;
		
		if (d = object_dictionaryarg (argc,argv))
			{
				x = (t_flap *)object_alloc (s_flap_class) ;
				boxflags = 0 
						| JBOX_DRAWFIRSTIN 
						| JBOX_NODRAWBOX
						| JBOX_DRAWINLAST
						| JBOX_TRANSPARENT	
						| JBOX_GROWY
						| JBOX_DRAWBACKGROUND
						| JBOX_HILITE  
						;

				jbox_new ((t_jbox *)x, boxflags, argc, argv) ;
				x->box.b_firstin = (void *)x ;
				
				x->m_out = bangout ((t_object *)x) ;
				
				defer_low (x, (method)flap_attach, NULL, 0, NULL) ;
				
				attr_dictionary_process (x, d) ;
				jbox_ready ((t_jbox *)x) ;
			}
		
		return x ;
	}

void flap_free (t_flap *x)
	{
		if (x->mypatcherview) 
			{
				object_detach_byptr (x, x->mypatcherview) ;
			}
			
		jbox_free ((t_jbox *)x) ;
	}

void flap_assist (t_flap *x, void *b, long m, long a, char *s)
	{
		if (m == ASSIST_INLET) 
			{ 
				sprintf (s, "(bang)") ;
			} 
		else 
			{	
				sprintf (s, "(bang)") ;
			}
	}

// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

void flap_paint (t_flap *x, t_object *patcherview)
	{
		t_rect		rect ;
		t_jgraphics *g = NULL ;
		
		g = (t_jgraphics*)patcherview_get_jgraphics (patcherview) ;		
		jbox_get_rect_for_view ((t_object *)x, patcherview, &rect) ;		
		
		jgraphics_set_source_jrgba (g, &x->couleur) ;
		jgraphics_rectangle (g, 0., 0., rect.width, rect.height) ;
		jgraphics_fill (g) ;
	}
	
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

void flap_getdrawparams (t_flap *x, t_object *patcherview, t_jboxdrawparams *params)
	{
		//params->d_bordercolor.alpha = 0 ;
		//params->d_boxfillcolor.alpha = 0 ;
	}
	
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

void flap_mousedown (t_flap *x, t_object *patcherview, t_pt pt, long modifiers)
	{	
		post ("!") ;
		// jbox_redraw ((t_jbox *)x) ;
	}

void flap_mousedrag (t_flap *x, t_object *patcherview, t_pt pt, long modifiers)
	{
		post ("!") ;
		// jbox_redraw ((t_jbox *)x) ;
	}

void flap_mouseup (t_flap *x, t_object *patcherview, t_pt pt, long modifiers)
	{
		post ("!") ;
		// jbox_redraw ((t_jbox *)x) ;
	}

void flap_mouseenter (t_flap *x, t_object *patcherview, t_pt pt, long modifiers)
	{
		post ("!") ;
		// jbox_redraw ((t_jbox *)x) ;	
	}

void flap_mouseleave(t_flap *x, t_object *patcherview, t_pt pt, long modifiers)
	{
		post ("!") ;
		// jbox_redraw ((t_jbox *)x) ;		
	}

void flap_mousemove (t_flap *x, t_object *patcherview, t_pt pt, long modifiers)
	{
		post ("!") ;
		// jbox_redraw ((t_jbox *)x) ;
	}

// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

void flap_key (t_flap *x, t_object *patcherview, long keycode, long modifiers, long textcharacter)
	{
		post ("!") ;
	}

void flap_focusgained (t_flap *x, t_object *patcherview)
	{
		post ("!") ;
	}

void flap_focuslost (t_flap *x, t_object *patcherview)
	{
		post ("!") ;
	}

// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

void flap_bang (t_flap *x)
	{
		outlet_bang (x->m_out) ;
	}

// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

void flap_attach (t_flap *x)
	{
		object_obex_lookup (x, gensym ("#P"), &x->mypatcher) ;

		x->mypatcherview = jpatcher_get_firstview (x->mypatcher) ;
		
		object_attach_byptr_register (x, x->mypatcherview, CLASS_NOBOX) ;
	}
	
void flap_notify (t_flap *x, t_symbol *s, t_symbol *msg, void *sender, void *data)
	{
		if (msg == gensym ("attr_modified") && sender == x->mypatcherview) 
			{ 			
				t_symbol *attr_name = NULL ;
				
				attr_name = (t_symbol *)object_method (data, gensym ("getname")) ;
				
				if (attr_name) 
					{
						post ("%s", attr_name->s_name) ;
					}
			}	
	}

// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------:x